import array, sys
# NanoJPEG -- KeyJ's Tiny Baseline JPEG Decoder
# version 1.1 (2010-03-05)
# by Martin J. Fiedler <martin.fiedler@gmx.net>
# http://keyj.emphy.de/nanojpeg/
#
# This software is published under the terms of KeyJ's Research License,
# version 0.2. Usage of this software is subject to the following conditions:
# 0. There's no warranty whatsoever. The author(s) of this software can not
#    be held liable for any damages that occur when using this software.
# 1. This software may be used freely for both non-commercial and commercial
#    purposes.
# 2. This software may be redistributed freely as long as no fees are charged
#    for the distribution and this license information is included.
# 3. This software may be modified freely except for this license information,
#    which must not be changed in any way.
# 4. If anything other than configuration, indentation or comments have been
#    altered in the code, the original author(s) must receive a copy of the
#    modified code.
#
# Ported to python by Andras Suller <suller.andras@gmail.com>


###############################################################################
## DOCUMENTATION SECTION                                                     ##
## read this if you want to know what this is all about                      ##
###############################################################################

# INTRODUCTION
# ============
#
# This is a minimal decoder for baseline JPEG images. It accepts memory dumps
# of JPEG files as input and generates either 8-bit grayscale or packed 24-bit
# RGB images as output. It does not parse JFIF or Exif headers; all JPEG files
# are assumed to be either grayscale or YCbCr. CMYK or other color spaces are
# not supported. All YCbCr subsampling schemes with power-of-two ratios are
# supported, as are restart intervals. Progressive or lossless JPEG is not
# supported.
# Summed up, NanoJPEG should be able to decode all images from digital cameras
# and most common forms of other non-progressive JPEG images.
# The decoder is not optimized for speed, it's optimized for simplicity and
# small code. Image quality should be at a reasonable level. A bicubic chroma
# upsampling filter ensures that subsampled YCbCr images are rendered in
# decent quality. The decoder is not meant to deal with broken JPEG files in
# a graceful manner; if anything is wrong with the bitstream, decoding will
# simply fail.
# The code should work with every modern C compiler without problems and
# should not emit any warnings. It uses only (at least) 32-bit integer
# arithmetic and is supposed to be endianness independent and 64-bit clean.
# However, it is not thread-safe.


# COMPILE-TIME CONFIGURATION
# ==========================
#
# The following aspects of NanoJPEG can be controlled with preprocessor
# defines:
#
# _NJ_EXAMPLE_PROGRAM     = Compile a main() function with an example
#                           program.
# _NJ_INCLUDE_HEADER_ONLY = Don't compile anything, just act as a header
#                           file for NanoJPEG. Example:
#                               #define _NJ_INCLUDE_HEADER_ONLY
#                               #include "nanojpeg.c"
#                               int main(void) {
#                                   njInit();
#                                   // your code here
#                                   njDone();
#                               }
# NJ_USE_LIBC=1           = Use the malloc(), free(), memset() and memcpy()
#                           functions from the standard C library (default).
# NJ_USE_LIBC=0           = Don't use the standard C library. In this mode,
#                           external functions njAlloc(), njFreeMem(),
#                           njFillMem() and njCopyMem() need to be defined
#                           and implemented somewhere.
# NJ_USE_WIN32=0          = Normal mode (default).
# NJ_USE_WIN32=1          = If compiling with MSVC for Win32 and
#                           NJ_USE_LIBC=0, NanoJPEG will use its own
#                           implementations of the required C library
#                           functions (default if compiling with MSVC and
#                           NJ_USE_LIBC=0).
# NJ_CHROMA_FILTER=1      = Use the bicubic chroma upsampling filter
#                           (default).
# NJ_CHROMA_FILTER=0      = Use simple pixel repetition for chroma upsampling
#                           (bad quality, but faster and less code).


# API
# ===
#
# For API documentation, read the "header section" below.


# EXAMPLE
# =======
#
# A few pages below, you can find an example program that uses NanoJPEG to
# convert JPEG files into PGM or PPM. To compile it, use something like
#     gcc -O3 -D_NJ_EXAMPLE_PROGRAM -o nanojpeg nanojpeg.c
# You may also add -std=c99 -Wall -Wextra -pedantic -Werror, if you want :)


###############################################################################
## HEADER SECTION                                                            ##
## copy and pase this into nanojpeg.h if you want                            ##
###############################################################################

#ifndef _NANOJPEG_H
#define _NANOJPEG_H

# nj_result_t: Result codes for njDecode().
NJ_OK = 0           # no error, decoding successful
NJ_NO_JPEG = 1      # not a JPEG file
NJ_UNSUPPORTED = 2  # unsupported format
NJ_OUT_OF_MEM = 3   # out of memory
NJ_INTERNAL_ERR = 4 # internal error
NJ_SYNTAX_ERROR = 5 # syntax error
__NJ_FINISHED = 6   # used internally, will never be reported

# njInit: Initialize NanoJPEG.
# For safety reasons, this should be called at least one time before using
# using any of the other NanoJPEG functions.
#void njInit(void);

# njDecode: Decode a JPEG image.
# Decodes a memory dump of a JPEG file into internal buffers.
# Parameters:
#   jpeg = The pointer to the memory dump.
#   size = The size of the JPEG file.
# Return value: The error code in case of failure, or NJ_OK (zero) on success.
#nj_result_t njDecode(const void* jpeg, const int size);

# njGetWidth: Return the width (in pixels) of the most recently decoded
# image. If njDecode() failed, the result of njGetWidth() is undefined.
#int njGetWidth(void);

# njGetHeight: Return the height (in pixels) of the most recently decoded
# image. If njDecode() failed, the result of njGetHeight() is undefined.
#int njGetHeight(void);

# njIsColor: Return 1 if the most recently decoded image is a color image
# (RGB) or 0 if it is a grayscale image. If njDecode() failed, the result
# of njGetWidth() is undefined.
#int njIsColor(void);

# njGetImage: Returns the decoded image data.
# Returns a pointer to the most recently image. The memory layout it byte-
# oriented, top-down, without any padding between lines. Pixels of color
# images will be stored as three consecutive bytes for the red, green and
# blue channels. This data format is thus compatible with the PGM or PPM
# file formats and the OpenGL texture formats GL_LUMINANCE8 or GL_RGB8.
# If njDecode() failed, the result of njGetImage() is undefined.
#unsigned char* njGetImage(void);

# njGetImageSize: Returns the size (in bytes) of the image data returned
# by njGetImage(). If njDecode() failed, the result of njGetImageSize() is
# undefined.
#int njGetImageSize(void);

# njDone: Uninitialize NanoJPEG.
# Resets NanoJPEG's internal state and frees all memory that has been
# allocated at run-time by NanoJPEG. It is still possible to decode another
# image after a njDone() call.
#void njDone(void);

#endif//_NANOJPEG_H


###############################################################################
## CONFIGURATION SECTION                                                     ##
## adjust the default settings for the NJ_ defines here                      ##
###############################################################################

NJ_USE_LIBC = 1

NJ_USE_WIN32 = 0

NJ_CHROMA_FILTER = 1


###############################################################################
## EXAMPLE PROGRAM                                                           ##
## just define _NJ_EXAMPLE_PROGRAM to compile this (requires NJ_USE_LIBC)    ##
###############################################################################

# #ifdef  _NJ_EXAMPLE_PROGRAM

# #include <stdio.h>
# #include <stdlib.h>
# #include <string.h>

# int main(int argc, char* argv[]) {
#     int size;
#     char *buf;
#     FILE *f;

#     if (argc < 2) {
#         printf("Usage: %s <input.jpg> [<output.ppm>]\n", argv[0]);
#         return 2;
#     }
#     f = fopen(argv[1], "rb");
#     if (!f) {
#         printf("Error opening the input file.\n");
#         return 1;
#     }
#     fseek(f, 0, SEEK_END);
#     size = (int) ftell(f);
#     buf = malloc(size);
#     fseek(f, 0, SEEK_SET);
#     size = (int) fread(buf, 1, size, f);
#     fclose(f);

#     njInit();
#     if (njDecode(buf, size)) {
#         printf("Error decoding the input file.\n");
#         return 1;
#     }

#     f = fopen((argc > 2) ? argv[2] : (njIsColor() ? "nanojpeg_out.ppm" : "nanojpeg_out.pgm"), "wb");
#     if (!f) {
#         printf("Error opening the output file.\n");
#         return 1;
#     }
#     fprintf(f, "P%d\n%d %d\n255\n", njIsColor() ? 6 : 5, njGetWidth(), njGetHeight());
#     fwrite(njGetImage(), 1, njGetImageSize(), f);
#     fclose(f);
#     njDone();
#     return 0;
# }

#endif


###############################################################################
## IMPLEMENTATION SECTION                                                    ##
## you may stop reading here                                                 ##
###############################################################################

#ifndef _NJ_INCLUDE_HEADER_ONLY

# typedef struct _nj_code {
#     unsigned char bits, code;
# } nj_vlc_code_t;

class nj_vlc_code_t(object):
    def __init__(self):
        self.bits = 0
        self.code = 0

# typedef struct _nj_cmp {
#     int cid;
#     int ssx, ssy;
#     int width, height;
#     int stride;
#     int qtsel;
#     int actabsel, dctabsel;
#     int dcpred;
#     unsigned char *pixels;
# } nj_component_t;

class nj_component_t(object):
    def __init__(self):
        self.cid = 0
        self.ssx = 0
        self.ssy= 0
        self.width = 0
        self.height = 0
        self.stride = 0
        self.qtsel = 0
        self.actabsel = 0
        self.dctabsel = 0
        self.dcpred = 0
        self.pixels = None

# typedef struct _nj_ctx {
#     nj_result_t error;
#     const unsigned char *pos;
#     int size;
#     int length;
#     int width, height;
#     int mbwidth, mbheight;
#     int mbsizex, mbsizey;
#     int ncomp;
#     nj_component_t comp[3];
#     int qtused, qtavail;
#     unsigned char qtab[4][64];
#     nj_vlc_code_t vlctab[4][65536];
#     int buf, bufbits;
#     int block[64];
#     int rstinterval;
#     unsigned char *rgb;
# } nj_context_t;

class nj_context_t(object):
    def init(self):
        self.error = 0
        self.spos = None # new param. it stores the string what is indexed by pos
        self.pos = 0
        self.size = 0
        self.length = 0
        self.width = 0
        self.height = 0
        self.mbwidth = 0
        self.mbheight = 0
        self.mbsizex = 0
        self.mbsizey = 0
        self.ncomp = 0
        self.comp = [nj_component_t(), nj_component_t(), nj_component_t()]
        self.qtused = 0
        self.qtavail = 0
        self.qtab = [[0] * 64, [0] * 64, [0] * 64, [0] * 64]
        # nj_vlc_code_t vlctab[4][65536] = None
        self.vlctab = []
        for n in range(4):
            self.vlctab.append([nj_vlc_code_t() for i in range(65536)])
        self.buf = 0
        self.bufbits = 0
        self.block = [0] * 64
        self.rstinterval = 0
        self.rgb = None

# static nj_context_t nj;
nj = nj_context_t()

njZZ = [ 0, 1, 8, 16, 9, 2, 3, 10, 17, 24, 32, 25, 18,
    11, 4, 5, 12, 19, 26, 33, 40, 48, 41, 34, 27, 20, 13, 6, 7, 14, 21, 28, 35,
    42, 49, 56, 57, 50, 43, 36, 29, 22, 15, 23, 30, 37, 44, 51, 58, 59, 52, 45,
    38, 31, 39, 46, 53, 60, 61, 54, 47, 55, 62, 63 ]

def njClip(x):
    if x < 0: return 0
    if x > 0xFF: return 0xFF
    return x

W1 = 2841
W2 = 2676
W3 = 2408
W5 = 1609
W6 = 1108
W7 = 565

def njRowIDCT(blk, p):
    x1 = blk[p + 4] << 11
    x2 = blk[p + 6]
    x3 = blk[p + 2]
    x4 = blk[p + 1]
    x5 = blk[p + 7]
    x6 = blk[p + 5]
    x7 = blk[p + 3]
    if (not (x1 | x2 | x3 | x4 | x5 | x6 | x7)):
        v = blk[p + 0] << 3
        blk[p + 0] = v
        blk[p + 1] = v
        blk[p + 2] = v
        blk[p + 3] = v
        blk[p + 4] = v
        blk[p + 5] = v
        blk[p + 6] = v
        blk[p + 7] = v
        return
    x0 = (blk[p + 0] << 11) + 128
    x8 = W7 * (x4 + x5)
    x4 = x8 + (W1 - W7) * x4
    x5 = x8 - (W1 + W7) * x5
    x8 = W3 * (x6 + x7)
    x6 = x8 - (W3 - W5) * x6
    x7 = x8 - (W3 + W5) * x7
    x8 = x0 + x1
    x0 -= x1
    x1 = W6 * (x3 + x2)
    x2 = x1 - (W2 + W6) * x2
    x3 = x1 + (W2 - W6) * x3
    x1 = x4 + x6
    x4 -= x6
    x6 = x5 + x7
    x5 -= x7
    x7 = x8 + x3
    x8 -= x3
    x3 = x0 + x2
    x0 -= x2
    x2 = (181 * (x4 + x5) + 128) >> 8
    x4 = (181 * (x4 - x5) + 128) >> 8
    blk[p + 0] = (x7 + x1) >> 8
    blk[p + 1] = (x3 + x2) >> 8
    blk[p + 2] = (x0 + x4) >> 8
    blk[p + 3] = (x8 + x6) >> 8
    blk[p + 4] = (x8 - x6) >> 8
    blk[p + 5] = (x0 - x4) >> 8
    blk[p + 6] = (x3 - x2) >> 8
    blk[p + 7] = (x7 - x1) >> 8


#blk was a char *, but we need to use an array and an index instead.
#sout is an extra parameter: it is the string what we need to modify,
#and out is the position inside sout (index)
def njColIDCT(blk, p, sout, out, stride):
    x1 = blk[p + 8*4] << 8
    x2 = blk[p + 8*6]
    x3 = blk[p + 8*2]
    x4 = blk[p + 8*1]
    x5 = blk[p + 8*7]
    x6 = blk[p + 8*5]
    x7 = blk[p + 8*3]
    if (not (x1 | x2 | x3 | x4 | x5 | x6 | x7)):
        x1 = njClip(((blk[p + 0] + 32) >> 6) + 128)
        x0 = 8
        while x0:
            sout[out] = x1
            out += stride
            x0 -= 1
        return
    
    x0 = (blk[p + 0] << 8) + 8192
    x8 = W7 * (x4 + x5) + 4
    x4 = (x8 + (W1 - W7) * x4) >> 3
    x5 = (x8 - (W1 + W7) * x5) >> 3
    x8 = W3 * (x6 + x7) + 4
    x6 = (x8 - (W3 - W5) * x6) >> 3
    x7 = (x8 - (W3 + W5) * x7) >> 3
    x8 = x0 + x1
    x0 -= x1
    x1 = W6 * (x3 + x2) + 4
    x2 = (x1 - (W2 + W6) * x2) >> 3
    x3 = (x1 + (W2 - W6) * x3) >> 3
    x1 = x4 + x6
    x4 -= x6
    x6 = x5 + x7
    x5 -= x7
    x7 = x8 + x3
    x8 -= x3
    x3 = x0 + x2
    x0 -= x2
    x2 = (181 * (x4 + x5) + 128) >> 8
    x4 = (181 * (x4 - x5) + 128) >> 8
    sout[out] = njClip(((x7 + x1) >> 14) + 128)
    out += stride
    sout[out] = njClip(((x3 + x2) >> 14) + 128)
    out += stride
    sout[out] = njClip(((x0 + x4) >> 14) + 128)
    out += stride
    sout[out] = njClip(((x8 + x6) >> 14) + 128)
    out += stride
    sout[out] = njClip(((x8 - x6) >> 14) + 128)
    out += stride
    sout[out] = njClip(((x0 - x4) >> 14) + 128)
    out += stride
    sout[out] = njClip(((x3 - x2) >> 14) + 128)
    out += stride
    sout[out] = njClip(((x7 - x1) >> 14) + 128)


def njShowBits(bits):
    if (not bits): return 0
    while (nj.bufbits < bits):
        if (nj.size <= 0):
            nj.buf = (nj.buf << 8) | 0xFF
            nj.bufbits += 8
            continue
        
        newbyte = nj.spos[nj.pos]
        nj.pos += 1
        nj.size -= 1
        nj.bufbits += 8
        nj.buf = (nj.buf << 8) | newbyte
        if (newbyte == 0xFF):
            if (nj.size):
                marker = nj.spos[nj.pos]
                nj.pos += 1
                nj.size -= 1
                if marker == 0xD9:
                    nj.size = 0
                elif marker != 0:
                    if ((marker & 0xF8) != 0xD0):
                        raise Exception(NJ_SYNTAX_ERROR)
                    else:
                        nj.buf = (nj.buf << 8) | marker
                        nj.bufbits += 8
            else:
                raise Exception(NJ_SYNTAX_ERROR)
    nj.buf = nj.buf & ((1 << nj.bufbits) - 1)
    return (nj.buf >> (nj.bufbits - bits)) & ((1 << bits) - 1)

def njSkipBits(bits):
    if (nj.bufbits < bits):
        njShowBits(bits)
    nj.bufbits -= bits

def njGetBits(bits):
    res = njShowBits(bits)
    njSkipBits(bits)
    return res

def njByteAlign():
    nj.bufbits &= 0xF8

def njSkip(count):
    nj.pos += count
    nj.size -= count
    nj.length -= count
    if (nj.size < 0): raise Exception(NJ_SYNTAX_ERROR)

def njDecode16(pos):
    return (nj.spos[pos] << 8) | nj.spos[pos + 1]

def njDecodeLength():
    if (nj.size < 2):
        raise Exception(NJ_SYNTAX_ERROR)
    nj.length = njDecode16(nj.pos)
    if (nj.length > nj.size):
        raise Exception(NJ_SYNTAX_ERROR)
    njSkip(2)

def njSkipMarker():
    njDecodeLength()
    njSkip(nj.length)

def njDecodeSOF():
    ssxmax = 0
    ssymax = 0
    njDecodeLength()
    if (nj.length < 9):
        raise Exception(NJ_SYNTAX_ERROR)
    if (nj.spos[nj.pos] != 8):
        raise Exception(NJ_UNSUPPORTED)
    nj.height = njDecode16(nj.pos + 1)
    nj.width = njDecode16(nj.pos + 3)
    nj.ncomp = nj.spos[nj.pos + 5]
    njSkip(6)
    if nj.ncomp != 1 and nj.ncomp != 3:
        raise Exception(NJ_UNSUPPORTED)
    if (nj.length < (nj.ncomp * 3)):
        raise Exception(NJ_SYNTAX_ERROR)
    i = 0
    while i < nj.ncomp:
        c = nj.comp[i]
        c.cid = nj.spos[nj.pos]
        c.ssx = nj.spos[nj.pos + 1] >> 4
        if not c.ssx:
            raise Exception(NJ_SYNTAX_ERROR)
        if (c.ssx & (c.ssx - 1)):
            raise Exception(NJ_UNSUPPORTED)  # non-power of two
        c.ssy = nj.spos[nj.pos + 1] & 15
        if not c.ssy:
            raise Exception(NJ_SYNTAX_ERROR)
        if (c.ssy & (c.ssy - 1)):
            raise Exception(NJ_UNSUPPORTED)  # non-power of two
        c.qtsel = nj.spos[nj.pos + 2]
        if c.qtsel & 0xFC:
            raise Exception(NJ_SYNTAX_ERROR)
        njSkip(3)
        nj.qtused |= 1 << c.qtsel
        if (c.ssx > ssxmax): ssxmax = c.ssx
        if (c.ssy > ssymax): ssymax = c.ssy
        i += 1
    
    nj.mbsizex = ssxmax << 3
    nj.mbsizey = ssymax << 3
    nj.mbwidth = (nj.width + nj.mbsizex - 1) // nj.mbsizex
    nj.mbheight = (nj.height + nj.mbsizey - 1) // nj.mbsizey
    i = 0
    while i < nj.ncomp:
        c = nj.comp[i]
        c.width = (nj.width * c.ssx + ssxmax - 1) // ssxmax
        c.stride = (c.width + 7)
        c.stride = c.stride & 0x7FFFFFF8
        c.height = (nj.height * c.ssy + ssymax - 1) // ssymax
        c.stride = nj.mbwidth * nj.mbsizex * c.ssx // ssxmax
        if (((c.width < 3) and (c.ssx != ssxmax)) or ((c.height < 3) and (c.ssy != ssymax))):
                raise Exception(NJ_UNSUPPORTED)
        c.pixels = [0] * (c.stride * (nj.mbheight * nj.mbsizey * c.ssy // ssymax))
        i += 1
    if (nj.ncomp == 3):
        nj.rgb = [0] * (nj.width * nj.height * nj.ncomp)
    njSkip(nj.length)

def njDecodeDHT():
    counts = [0] * 16
    njDecodeLength()
    while (nj.length >= 17):
        i = nj.spos[nj.pos]
        if (i & 0xEC):
            raise Exception(NJ_SYNTAX_ERROR)
        if (i & 0x02):
            raise Exception(NJ_UNSUPPORTED)
        i = (i | (i >> 3)) & 3  # combined DC/AC + tableid value
        for codelen in range(1, 17): # 1 to 16
            counts[codelen - 1] = nj.spos[nj.pos + codelen]
        njSkip(17)
        vlc = 0
        remain = 65536
        spread = 65536
        for codelen in range(1, 17): # 1 to 16
            spread >>= 1
            currcnt = counts[codelen - 1]
            if not currcnt: continue
            if (nj.length < currcnt):
                raise Exception(NJ_SYNTAX_ERROR)
            remain -= currcnt << (16 - codelen)
            if (remain < 0):
                raise Exception(NJ_SYNTAX_ERROR)
            for ii in range(currcnt):
                code = nj.spos[nj.pos + ii]
                j = spread
                while j:
                    nj.vlctab[i][vlc].bits = codelen
                    nj.vlctab[i][vlc].code = code
                    vlc += 1
                    j -= 1
            njSkip(currcnt)
        while remain:
            remain -= 1
            nj.vlctab[i][vlc].bits = 0
            vlc += 1
    if (nj.length):
        raise Exception(NJ_SYNTAX_ERROR)

def njDecodeDQT():
    njDecodeLength()
    while (nj.length >= 65):
        i = nj.spos[nj.pos]
        if (i & 0xFC):
            raise Exception(NJ_SYNTAX_ERROR)
        nj.qtavail |= 1 << i
        for j in range(64):
            nj.qtab[i][j] = nj.spos[nj.pos + j + 1]
        njSkip(65)
    if (nj.length):
        raise Exception(NJ_SYNTAX_ERROR)

def njDecodeDRI():
    njDecodeLength()
    if (nj.length < 2):
        raise Exception(NJ_SYNTAX_ERROR)
    nj.rstinterval = njDecode16(nj.pos)
    njSkip(nj.length)

#code is an array with one element, since we need to return the code to the caller
def njGetVLC(vlc, code):
    value = njShowBits(16)
    bits = vlc[value].bits
    if not bits:
        raise Exception(NJ_SYNTAX_ERROR)
    njSkipBits(bits)
    value = vlc[value].code
    if code: code[0] = value
    bits = value & 15
    if not bits: return 0
    value = njGetBits(bits)
    if (value < (1 << (bits - 1))):
        value += ((-1) << bits) + 1
    return value

#sout is a new parameter, because we need to modify the passed in array, so
#out is now just the index in out
def njDecodeBlock(c, sout, out):
    code = [0]
    value = 0
    coef = 0
    for i in range(len(nj.block)):
        nj.block[i] = 0
    c.dcpred += njGetVLC(nj.vlctab[c.dctabsel], None)
    nj.block[0] = c.dcpred * nj.qtab[c.qtsel][0]
    while True: # do {
        value = njGetVLC(nj.vlctab[c.actabsel], code);
        if not code[0]: break  # EOB
        if (not (code[0] & 0x0F) and (code[0] != 0xF0)):
            raise Exception(NJ_SYNTAX_ERROR)
        coef += (code[0] >> 4) + 1
        if coef > 63:
            raise Exception(NJ_SYNTAX_ERROR)
        nj.block[njZZ[coef]] = value * nj.qtab[c.qtsel][coef]
        # } while (coef < 63);
        if coef >= 63: break
    coef = 0
    while coef < 64:
        njRowIDCT(nj.block, coef)
        coef += 8
    for coef in range(8):
        njColIDCT(nj.block, coef, sout, out + coef, c.stride)

def njDecodeScan():
    rstcount = nj.rstinterval
    nextrst = 0
    # nj_component_t* c;
    njDecodeLength()
    if (nj.length < (4 + 2 * nj.ncomp)):
        raise Exception(NJ_SYNTAX_ERROR)
    if (nj.spos[nj.pos] != nj.ncomp):
        raise Exception(NJ_UNSUPPORTED)
    njSkip(1)
    i = 0
    while (i < nj.ncomp):
        c = nj.comp[i]
        if (nj.spos[nj.pos] != c.cid):
            raise Exception(NJ_SYNTAX_ERROR)
        if (nj.spos[nj.pos + 1] & 0xEE):
            raise Exception(NJ_SYNTAX_ERROR)
        c.dctabsel = nj.spos[nj.pos + 1] >> 4
        c.actabsel = (nj.spos[nj.pos + 1] & 1) | 2
        njSkip(2)
        i += 1
    if (nj.spos[nj.pos] or (nj.spos[nj.pos + 1] != 63) or nj.spos[nj.pos + 2]):
        raise Exception(NJ_UNSUPPORTED)
    njSkip(nj.length)
    mbx = 0
    mby = 0
    while True:
        i = 0
        while (i < nj.ncomp):
            c = nj.comp[i]
            sby = 0
            while sby < c.ssy:
                sbx = 0
                while sbx < c.ssx:
                    njDecodeBlock(c, c.pixels, ((mby * c.ssy + sby) * c.stride + mbx * c.ssx + sbx) << 3)
                    if nj.error:
                        return
                    sbx += 1
                sby += 1
            i += 1
        mbx += 1
        if mbx >= nj.mbwidth:
            mbx = 0
            mby += 1
            if mby >= nj.mbheight: break
        rstcount -= 1
        if (nj.rstinterval and not rstcount):
            njByteAlign()
            i = njGetBits(16)
            if (((i & 0xFFF8) != 0xFFD0) or ((i & 7) != nextrst)):
                raise Exception(NJ_SYNTAX_ERROR)
            nextrst = (nextrst + 1) & 7
            rstcount = nj.rstinterval
            for i in range(3):
                nj.comp[i].dcpred = 0
    nj.error = __NJ_FINISHED

#if NJ_CHROMA_FILTER

CF4A = -9
CF4B = 111
CF4C = 29
CF4D = -3
CF3A = 28
CF3B = 109
CF3C = -9
CF3X = 104
CF3Y = 27
CF3Z = -3
CF2A = 139
CF2B = -11
def CF(x):
    return njClip(((x) + 64) >> 7)

def njUpsampleH(c):
    xmax = c.width - 3
    out = [0] * ((c.width * c.height) << 1)
    lin = 0
    lout = 0
    y = c.height
    while y:
        out[lout + 0] = CF(CF2A * c.pixels[lin + 0] + CF2B * c.pixels[lin + 1])
        out[lout + 1] = CF(CF3X * c.pixels[lin + 0] + CF3Y * c.pixels[lin + 1] + CF3Z * c.pixels[lin + 2])
        out[lout + 2] = CF(CF3A * c.pixels[lin + 0] + CF3B * c.pixels[lin + 1] + CF3C * c.pixels[lin + 2])
        for x in range(xmax):
            out[lout + (x << 1) + 3] = CF(CF4A * c.pixels[lin + x] + CF4B * c.pixels[lin + x + 1] + CF4C * c.pixels[lin + x + 2] + CF4D * c.pixels[lin + x + 3])
            out[lout + (x << 1) + 4] = CF(CF4D * c.pixels[lin + x] + CF4C * c.pixels[lin + x + 1] + CF4B * c.pixels[lin + x + 2] + CF4A * c.pixels[lin + x + 3])
        lin += c.stride
        lout += c.width << 1
        out[lout - 3] = CF(CF3A * c.pixels[lin - 1] + CF3B * c.pixels[lin - 2] + CF3C * c.pixels[lin - 3])
        out[lout - 2] = CF(CF3X * c.pixels[lin - 1] + CF3Y * c.pixels[lin - 2] + CF3Z * c.pixels[lin - 3])
        out[lout - 1] = CF(CF2A * c.pixels[lin - 1] + CF2B * c.pixels[lin - 2])
        y -= 1
    c.width <<= 1
    c.stride = c.width
    c.pixels = out

def njUpsampleV(c):
    w = c.width
    s1 = c.stride
    s2 = s1 + s1
    out = [0] * ((c.width * c.height) << 1)
    for x in range(w):
        cin = x
        cout = x
        out[cout] = CF(CF2A * c.pixels[cin] + CF2B * c.pixels[cin + s1])
        cout += w
        out[cout] = CF(CF3X * c.pixels[cin] + CF3Y * c.pixels[cin + s1] + CF3Z * c.pixels[cin + s2])
        cout += w
        out[cout] = CF(CF3A * c.pixels[cin] + CF3B * c.pixels[cin + s1] + CF3C * c.pixels[cin + s2])
        cout += w
        cin += s1
        y = c.height - 3
        while y:
            out[cout] = CF(CF4A * c.pixels[cin - s1] + CF4B * c.pixels[cin] + CF4C * c.pixels[cin + s1] + CF4D * c.pixels[cin + s2])
            cout += w
            out[cout] = CF(CF4D * c.pixels[cin - s1] + CF4C * c.pixels[cin] + CF4B * c.pixels[cin + s1] + CF4A * c.pixels[cin + s2])
            cout += w
            cin += s1
            y -= 1
        cin += s1
        out[cout] = CF(CF3A * c.pixels[cin] + CF3B * c.pixels[cin - s1] + CF3C * c.pixels[cin - s2])
        cout += w
        out[cout] = CF(CF3X * c.pixels[cin] + CF3Y * c.pixels[cin - s1] + CF3Z * c.pixels[cin - s2])
        cout += w
        out[cout] = CF(CF2A * c.pixels[cin] + CF2B * c.pixels[cin - s1])
    c.height <<= 1
    c.stride = c.width
    c.pixels = out

#else

# NJ_INLINE void njUpsample(nj_component_t* c) {
#     int x, y, xshift = 0, yshift = 0;
#     unsigned char *out, *lin, *lout;
#     while (c->width < nj.width) { c->width <<= 1; ++xshift; }
#     while (c->height < nj.height) { c->height <<= 1; ++yshift; }
#     out = njAllocMem(c->width * c->height);
#     if (!out) njThrow(NJ_OUT_OF_MEM);
#     lin = c->pixels;
#     lout = out;
#     for (y = 0;  y < c->height;  ++y) {
#         lin = &c->pixels[(y >> yshift) * c->stride];
#         for (x = 0;  x < c->width;  ++x)
#             lout[x] = lin[x >> xshift];
#         lout += c->width;
#     }
#     c->stride = c->width;
#     njFreeMem(c->pixels);
#     c->pixels = out;
# }

#endif

def njConvert():
    for i in range(nj.ncomp):
        c = nj.comp[i]
        if NJ_CHROMA_FILTER:
            while ((c.width < nj.width) or (c.height < nj.height)):
                if c.width < nj.width: njUpsampleH(c)
                if nj.error: return
                if c.height < nj.height: njUpsampleV(c)
                if nj.error: return
        else:
            if ((c.width < nj.width) or (c.height < nj.height)):
                njUpsample(c)
        if ((c.width < nj.width) or (c.height < nj.height)):
            raise Exception(NJ_INTERNAL_ERR)
            return
    if nj.ncomp == 3:
        # convert to RGB
        prgb = 0
        py  = 0
        pcb = 0
        pcr = 0
        yy = nj.height
        #print( 'nj.width: %s, nj.height: %s, strides: %s, %s, %s, lengths: %s, %s, %s, first few bytes: %s, %s, %s' % \
        #    (nj.width, nj.height, nj.comp[0].stride, nj.comp[1].stride, nj.comp[2].stride,
        #    len(nj.comp[0].pixels), len(nj.comp[1].pixels), len(nj.comp[2].pixels),
        #    nj.comp[0].pixels[0], nj.comp[0].pixels[1], nj.comp[0].pixels[2]))
        while yy:
            for x in range(nj.width):
                y = nj.comp[0].pixels[py + x] << 8
                # print 'len(nj.comp): %s, len(nj.comp[1].pixels): %s, pcb + x: %s, yy: %s, x: %s' % \
                #     (len(nj.comp), len(nj.comp[1].pixels), pcb + x, yy, x)
                cb = nj.comp[1].pixels[pcb + x] - 128
                cr = nj.comp[2].pixels[pcr + x] - 128
                nj.rgb[prgb] = njClip((y            + 359 * cr + 128) >> 8)
                prgb += 1
                nj.rgb[prgb] = njClip((y -  88 * cb - 183 * cr + 128) >> 8)
                prgb += 1
                nj.rgb[prgb] = njClip((y + 454 * cb            + 128) >> 8)
                prgb += 1
            py += nj.comp[0].stride
            pcb += nj.comp[1].stride
            pcr += nj.comp[2].stride
            yy -= 1
    elif (nj.comp[0].width != nj.comp[0].stride):
        # grayscale -> only remove stride
        # unsigned char *pin = &nj.comp[0].pixels[nj.comp[0].stride];
        # unsigned char *pout = &nj.comp[0].pixels[nj.comp[0].width];
        # int y;
        # for (y = nj.comp[0].height - 1;  y;  --y) {
        #     njCopyMem(pout, pin, nj.comp[0].width);
        #     pin += nj.comp[0].stride;
        #     pout += nj.comp[0].width;
        # }
        # nj.comp[0].stride = nj.comp[0].width;
        pass

def njInit():
    # njFillMem(&nj, 0, sizeof(nj_context_t));
    nj.init()

def njDone():
    pass

def njDecode(jpeg, size):
    njDone()
    nj.spos = jpeg
    nj.pos = 0
    nj.size = size & 0x7FFFFFFF
    if (nj.size < 2): return NJ_NO_JPEG
    if ((nj.spos[nj.pos] ^ 0xFF) | (nj.spos[nj.pos + 1] ^ 0xD8)): return NJ_NO_JPEG
    njSkip(2)
    while not nj.error:
        if ((nj.size < 2) or (nj.spos[nj.pos] != 0xFF)):
            return NJ_SYNTAX_ERROR
        njSkip(2)
        m = nj.spos[nj.pos - 1]
        if m == 0xC0: njDecodeSOF()
        elif m == 0xC4: njDecodeDHT()
        elif m == 0xDB: njDecodeDQT()
        elif m == 0xDD: njDecodeDRI()
        elif m == 0xDA: njDecodeScan()
        elif m == 0xFE: njSkipMarker()
        elif (m & 0xF0) == 0xE0:
            njSkipMarker()
        else:
            return NJ_UNSUPPORTED
    if (nj.error != __NJ_FINISHED): return nj.error
    nj.error = NJ_OK
    njConvert()
    return nj.error


def njGetWidth():
    return nj.width
def njGetHeight():
    return nj.height
def njIsColor():
    return (nj.ncomp != 1)
def njGetImage():
    return nj.comp[0].pixels if nj.ncomp == 1 else nj.rgb
def njGetImageSize():
    return nj.width * nj.height * nj.ncomp

#endif // _NJ_INCLUDE_HEADER_ONLY



# http://stackoverflow.com/questions/14267900/python-drag-and-drop-explorer-files-to-tkinter-entry-widget/
import os
try: import tkinter
except: import Tkinter as tkinter

def _load_tkdnd(master):
    tkdndlib = os.environ.get('TKDND_LIBRARY')
    if tkdndlib:
        master.tk.eval('global auto_path; lappend auto_path {%s}' % tkdndlib)
    master.tk.eval('package require tkdnd')
    master._tkdnd_loaded = True
    
class TkDND(object):
    def __init__(self, master):
        if not getattr(master, '_tkdnd_loaded', False):
            _load_tkdnd(master)
        self.master = master
        self.tk = master.tk

    # Available pre-defined values for the 'dndtype' parameter:
    #   text/plain
    #   text/plain;charset=UTF-8
    #   text/uri-list

    def bindtarget(self, window, callback, dndtype, event='<Drop>', priority=50):
        cmd = self._prepare_tkdnd_func(callback)
        return self.tk.call('dnd', 'bindtarget', window, dndtype, event, cmd, priority)

    def bindtarget_query(self, window, dndtype=None, event='<Drop>'):
        return self.tk.call('dnd', 'bindtarget', window, dndtype, event)

    def cleartarget(self, window):
        self.tk.call('dnd', 'cleartarget', window)

    def bindsource(self, window, callback, dndtype, priority=50):
        cmd = self._prepare_tkdnd_func(callback)
        self.tk.call('dnd', 'bindsource', window, dndtype, cmd, priority)

    def bindsource_query(self, window, dndtype=None):
        return self.tk.call('dnd', 'bindsource', window, dndtype)

    def clearsource(self, window):
        self.tk.call('dnd', 'clearsource', window)

    def drag(self, window, actions=None, descriptions=None,
            cursorwin=None, callback=None):
        cmd = None
        if cursorwin is not None:
            if callback is not None:
                cmd = self._prepare_tkdnd_func(callback)
        self.tk.call('dnd', 'drag', window, actions, descriptions,
                cursorwin, cmd)


    _subst_format = ('%A', '%a', '%b', '%D', '%d', '%m', '%T',
            '%W', '%X', '%Y', '%x', '%y')
    _subst_format_str = " ".join(_subst_format)

    def _prepare_tkdnd_func(self, callback):
        funcid = self.master.register(callback, self._dndsubstitute)
        cmd = ('%s %s' % (funcid, self._subst_format_str))
        return cmd

    def _dndsubstitute(self, *args):
        if len(args) != len(self._subst_format):
            return args

        def try_int(x):
            x = str(x)
            try:
                return int(x)
            except ValueError:
                return x

        A, a, b, D, d, m, T, W, X, Y, x, y = args

        event = tkinter.Event()
        event.action = A       # Current action of the drag and drop operation.
        event.action_list = a  # Action list supported by the drag source.
        event.mouse_button = b # Mouse button pressed during the drag and drop.
        event.data = D         # The data that has been dropped.
        event.descr = d        # The list of descriptions.
        event.modifier = m     # The list of modifier keyboard keys pressed.
        event.dndtype = T
        event.widget = self.master.nametowidget(W)
        event.x_root = X       # Mouse pointer x coord, relative to the root win.
        event.y_root = Y
        event.x = x            # Mouse pointer x coord, relative to the widget.
        event.y = y

        event.action_list = str(event.action_list).split()
        for name in ('mouse_button', 'x', 'y', 'x_root', 'y_root'):
            setattr(event, name, try_int(getattr(event, name)))

        return (event, )



import sys
import time
sys.path.append("../../")
from appJar import gui

# global variable to remember what's being dragged
dragged = "dd"
# globals to remember the meter's values
meter1 = 0
meter2 = 100
# some useful colours
colours = ["red", "orange", "green", "pink", "purple"]

# calculator function
calcVal = 0
def calculator(key):
    global calcVal
    val = app.getLabel("calculator")
    app.setLabel("calculator", val + key)

# function to login the user
def login(btn):
    if btn == "Clear":
        app.clearEntry("username")
        app.clearEntry("password")
        app.setEntryFocus("username")
    elif btn == "Submit":
        app.infoBox("Success", "Access granted")
        app.setTabbedFrameDisableAllTabs("Tabs", False)
        app.setToolbarEnabled()
        app.setToolbarPinned()
        app.setToolbarButtonDisabled("LOGOUT", False)
        app.enableMenubar()

# function to confirm logout
def logoutFunction():
    return app.yesNoBox("Confirm Exit", "Are you sure you want to exit?")

# disable the tabs
def logout(btn = None):
    app.setTabbedFrameDisableAllTabs("Tabs")
    app.setTabbedFrameDisabledTab("Tabs", "Login", False)
    app.setTabbedFrameSelectedTab("Tabs", "Login")
    # disable toolbar
    app.setToolbarDisabled()
    app.setToolbarButtonEnabled("EXIT")
    app.disableMenubar()
    app.enableMenuItem("Test", "EXIT")

# function to update the meters
def meters():
    global meter1, meter2
    meter1=(meter1+1)%100
    meter2=(meter2-1)%100
    app.setMeter("Meter1", meter1)
    app.setMeter("Meter2", meter2)

# functions to handle drag'n drop

# call this to reset the drag'n drop label
def resetDD(btn):
    app.setLabel("dd", "Drop here")
    app.setLabelBg("dd", "blue")

# called when a drag starts - remember the label being dragged
def drag(lbl):
    global dragged
    dragged = lbl

# called when a drag stops - check the label being dropped on
def drop(lbl):
    if lbl == "dd": 
        app.setLabel("dd", app.getLabel(dragged))
        app.setLabelBg("dd", colours[int(dragged[2])])

# called by the toolbar buttons
def toolbar(btn):
    print(btn)
    if btn == "EXIT": app.stop()
    elif btn == "LOGOUT": logout()
    elif btn == "FILL": app.setTabBg("Tabs", app.getTabbedFrameSelectedTab("Tabs"), app.colourBox())
    elif btn == "PIE-CHART": app.showSubWindow("Statistics")
    elif btn == "FULL-SCREEN":
        if app.exitFullscreen():
            app.setToolbarIcon("FULL-SCREEN", "FULL-SCREEN")
        else:
            app.setGeometry("fullscreen")
            app.setToolbarIcon("FULL-SCREEN", "FULL-SCREEN-EXIT")
    elif btn == "CALENDAR": app.showSubWindow("DatePicker")
    elif btn == "ADDRESS-BOOK": app.showSubWindow("AddressBook")
    elif btn == "MAP": app.showSubWindow("Maps")

# called when scale/meters are changed
def scale(name):
    if name == "TransparencySpin":
        trans = int(app.getSpinBox(name))
        app.setTransparency(trans)
        app.setScale("TransparencyScale", trans, callFunction=False)
    elif name == "TransparencyScale":
        trans = app.getScale(name)
        app.setTransparency(trans)
        app.setSpinBox("TransparencySpin", trans, callFunction=False)
    elif name == "FontScale": app.setFont(app.getScale(name))

def move(direction):
    if direction == ">":
        for item in app.getListBox("Animals"):
            app.addListItem("Sports",item) 
            app.removeListItem("Animals", item)
    elif direction == "<":
        for item in app.getListBox("Sports"):
            app.addListItem("Animals",item) 
            app.removeListItem("Sports", item)
    elif direction == "<<":
        app.addListItems("Animals", app.getAllListItems("Sports"))
        app.clearListBox("Sports")
    elif direction == ">>":
        app.addListItems("Sports", app.getAllListItems("Animals"))
        app.clearListBox("Animals")

def add(entry):
    if entry == "animalsEntry":
        app.addListItem("Animals", app.getEntry("animalsEntry"))
        app.clearEntry("animalsEntry")
    elif entry == "sportsEntry":
        app.addListItem("Sports", app.getEntry("sportsEntry"))
        app.clearEntry("sportsEntry")

# funciton to change the selected tab - called from menu
def changeTab(tabName):
    print("Changing to: ", tabName)
    app.setTabbedFrameSelectedTab("Tabs", tabName)
    print("done")

# function to get a help message on log-in page
def helpMe(nbtn):
    app.infoBox("Login Help", "Any username/password will do...")

# function to update status bar with the time
def showTime():
    app.setStatusbar(time.strftime("%X"))

###########################
## GUI Code starts here ##
###########################

with gui("ShowCase") as app:
    app.showSplash("appJar Showcase")

    # add a simple toolbar
    app.addToolbar(["EXIT", "LOGOUT", "FILL", "PIE-CHART", "CALENDAR", "ADDRESS-BOOK", "MAP", "FULL-SCREEN"], toolbar, findIcon=True)

    #app.createMenu("Test")
    app.addMenuPreferences(toolbar)
    #app.addMenuItem("APPMENU", "About", toolbar)
    app.addMenuItem("Test", "EXIT", toolbar, shortcut="Option-Control-Shift-Alt-Command-B", underline=2)
    app.addMenuList("Tabs", ["Login", "Lists", "Properties", "Meters", "Drag'nDrop", "Calculator", "Panes", "Labels"], changeTab)
    app.addMenuItem("Test", "LOGOUT", toolbar, shortcut="Shift-Command-B", underline=3)
    app.addMenuItem("Test", "FILL", toolbar, shortcut="Control-Shift-C", underline=1)
    app.addSubMenu("Test", "Bobs")
    app.addMenuItem("Bobs", "EXIT", toolbar)
    app.addMenuItem("Bobs", "LOGOUT", toolbar)
    app.addMenuItem("Bobs", "FILL", toolbar)
    app.addMenu("PRESSME", toolbar)
    app.addMenuCheckBox("Boxes", "Box 1", toolbar, "Command-1")
    app.addMenuCheckBox("Boxes", "Box 2", toolbar, "Command-2")
    app.addMenuCheckBox("Boxes", "Box 3", toolbar, "Command-3")
    app.addMenuCheckBox("Boxes", "Box 4", toolbar, "Command-4")
    app.addMenuCheckBox("Boxes", "Box 5", toolbar, "Command-5")

    app.addMenuRadioButton("Radios", "r1", "Radio 6", toolbar, "Command-6", 7)
    app.addMenuRadioButton("Radios", "r1", "Radio 7", toolbar, "Command-7", 7)
    app.addMenuRadioButton("Radios", "r1", "Radio 8", toolbar, "Command-8", 7)
    app.addMenuRadioButton("Radios", "r1", "Radio 9", toolbar, "Command-9", 7)
    app.addMenuRadioButton("Radios", "r1", "Radio 0", toolbar, "Command-0", 7)

    app.addMenuList("List Items", ["Item 1", "Item 2", "-", "Item 3", "Item 4", "Item 5", "Item 6", "Item 7"], toolbar)
    app.addMenuList("List Items", ["-", "aaa", "-", "bbb", "-", "ccc", "-", "ddd", "-"], toolbar)

    #app.addMenuItem("2222List Items", "FILL", toolbar, shortcut="Control-Shift-C")

    app.addMenuWindow()
    app.addMenuHelp(toolbar)
    app.addMenuItem("WIN_SYS", "About", app.appJarAbout)

    app.addMenuList("Help", ["Help", "About"], [app.appJarHelp, app.appJarAbout])

    try:
        app.setMenuIcon("Test", "EXIT", "EXIT", "left")
        app.setMenuIcon("Test", "LOGOUT", "LOGOUT", "right")
        app.setMenuIcon("Test", "FILL", "FILL", "none")
    except:
        pass

    app.disableMenuItem("List Items", "aaa")
    app.disableMenubar()

    # add a statusbar to show the time
    app.addStatusbar(side="RIGHT")
    app.registerEvent(showTime)

    app.setStopFunction(logoutFunction)

    # start the tabbed frame
    with app.tabbedFrame("Tabs"):
        #### LOGIN TAB
        with app.tab("Login"):
            app.setBg("SlateGray")
            app.setSticky("NEW")
            with app.labelFrame("Login Form"):
                app.setSticky("ew")
                app.addLabel("username", "Username", 0, 0)
                app.addEntry("username", 0, 1)
                app.addLabel("password", "Password", 1, 0)
                app.addSecretEntry("password", 1, 1)
                app.addButtons(["Submit", "Clear"], login, 2, 0, 2)
                app.setEntryFocus("username")
                app.setSticky("e")
                app.addLink("help", helpMe, column=1)

        #### LISTS TAB
        with app.tab("Lists"):
            app.setSticky("ew")
            app.setExpand("all")
            app.setBg("OrangeRed")

            app.addListBox("Animals", ["Zebra", "Sheep", "Lion", "Giraffe", "Snake", "Fish"], 0, 0, 1, 4)
            app.addListBox("Sports", ["Football", "Rugby", "Cricket", "Golf", "Cycling", "Netball", "Rounders"], 0, 2, 1, 4)

            app.addButton("<", move, 0, 1)
            app.addButton("<<", move, 1, 1)
            app.addButton(">>", move, 2, 1)
            app.addButton(">", move, 3, 1)

            app.addEntry("animalsEntry", 4, 0)
            app.addEntry("sportsEntry", 4, 2)

            app.setListBoxBg("Animals", "Orange")
            app.setListBoxBg("Sports", "Orange")
            app.setEntryBg("animalsEntry", "LightGrey")
            app.setEntryBg("sportsEntry", "LightGrey")
            app.setEntrySubmitFunction("animalsEntry", add)
            app.setEntrySubmitFunction("sportsEntry", add)

            app.setEntryDefault("animalsEntry", "-- add an item --")
            app.setEntryDefault("sportsEntry", "-- add an item --")

        #### PROPERTIES TAB
        with app.tab("Properties"):
            with app.panedFrame("left"):
                app.addLabel("t", "Transparency")
                app.addLabel("t2", "Transparency")
                app.addLabel("f","Font")

                with app.panedFrame("right"):
                    app.addScale("TransparencyScale")
                    app.setScale("TransparencyScale", 100)
                    app.setScaleChangeFunction("TransparencyScale", scale)
                    app.showScaleIntervals("TransparencyScale", 25)

                    app.addSpinBoxRange("TransparencySpin", 0, 100)
                    app.setSpinBox("TransparencySpin", 100)
                    app.setSpinBoxChangeFunction("TransparencySpin", scale)

                    app.addScale("FontScale")
                    app.setScaleRange("FontScale", 6, 40, 12)
                    app.setScaleChangeFunction("FontScale", scale)
                    app.showScaleValue("FontScale")

        #### METERS TAB
        with app.tab("Meters"):
            app.addMeter("Meter1")
            app.addMeter("Meter2")
            app.setMeterFill("Meter1", "Yellow")
            app.setMeterFill("Meter2", "Orange")
            app.registerEvent(meters)

        #### DRAG'N DROP TAB
        with app.tab("Drag'nDrop"):
            with app.labelFrame(""):
                app.setSticky("news")
                app.setIPadX(20)
                app.setIPadY(20)

                for i in range(5):
                    l = "DD"+str(i)
                    app.addLabel(l, l, 0, i)
                    app.setLabelBg(l, colours[i])
                    app.setLabelFg(l, "white")
                    app.setLabelDragFunction(l, [drag, drop])

                app.addHorizontalSeparator(1, 0, 5)

                app.addLabel("dd", "DROP HERE", 2, 0, 5)
                app.setLabelTooltip("dd", "Drag any of the colours here to make a change...")
                app.setLabelBg("dd", "blue")
                app.setLabelFg("dd", "white")
                app.stopLabelFrame()

                app.startLabelFrame("Reset")
                app.setSticky("news")
                app.addButton("RESET", resetDD, 0,0)
                app.addLabel("RESET", "RESET", 0,1)
                app.setLabelBg("RESET", "grey")
                app.setLabelSubmitFunction("RESET", resetDD)
                app.addLink("RESET", resetDD, 0,2)

        with app.tab("Calculator"):
            app.setIPadX(5)
            app.setIPadY(5)
            app.addEmptyLabel("calculator")
            app.setLabelBg("calculator", "grey")
            app.setLabelRelief("calculator", "sunken")
            app.setLabelAlign("calculator", "e")
            buttons=[["1", "2", "3", "C"],
                            ["4", "5", "6", "+"],
                            ["7", "8", "9", "-"],
                            ["0", "*", "/", "="]]

            app.addButtons(buttons, calculator)
            app.setButtonWidths(buttons, 3)
            app.setButtonHeights(buttons, 3)

        with app.tab("Panes"):
            with app.panedFrame("a"):
                app.setSticky("news")
                app.addLabel("p1", "Edit Pane")
                app.setLabelRelief("p1", "groove")
                app.addTextArea("t1")
                with app.panedFrameVertical("b"):
                    app.addLabel("p2", "Pane 2")
                with app.panedFrame("c"):
                    app.addLabel("p3", "Pane 3")
                with app.panedFrame("d"):
                    app.addLabel("p4", "Pane 4")
          
        with app.tab("Labels"):
            app.setSticky("nesw")

            app.addLabel("ll1", "Red", 0, 1, 4)
            app.setLabelBg("ll1", "Red")
            app.addLabel("ll2", "Yellow", 0, 0, 1, 4)
            app.setLabelBg("ll2", "Yellow")
            app.addLabel("ll3", "Green", 1, 1, 2)
            app.setLabelBg("ll3", "Green")
            app.addLabel("ll4", "Blue", 1, 3, 1, 2)
            app.setLabelBg("ll4", "Blue")
            app.addLabel("ll5", "Orange", 2, 1, 2)
            app.setLabelBg("ll5", "Orange")
            app.addLabel("ll6", "Pink", 3, 1, 4)
            app.setLabelBg("ll6", "Pink")
            app.addLabel("ll7", "Purple", 4, 0, 4)
            app.setLabelBg("ll7", "Purple")

    with app.subWindow("Statistics", transient=True):
        app.setBg("yellow")
        app.setSticky("news")
        app.setGeometry("300x330")
        values={"German":20, "French":10, "English":60, "Dutch": 5, "Belgium":3, "Danish":2}
        app.addPieChart("Nationality", values)
        app.addLabelOptionBox("Nationality", values.keys())
        app.addNumericEntry("Percentage")
        def changePie(btn):
            app.setPieChart("Nationality", app.getOptionBox("Nationality"), app.getEntry("Percentage"))
        app.addButton("Update", changePie)

    # google maps
    with app.subWindow("Maps"):
        app.setSticky("news")
        app.addGoogleMap("g1")

    # date picker
    with app.subWindow("DatePicker", transient=True, modal=True):
        app.addDatePicker("dp")
        app.setDatePicker("dp")
        def getDate(btn=None):
            print(app.getDatePicker("dp"))
            return True
        app.addNamedButton("DONE", "DatePicker", app.hideSubWindow)
        app.setStopFunction(getDate)

    # paged window
    with app.subWindow("AddressBook", transient=True):
        with app.pagedWindow("AddressBook"):
            with app.page():
                app.addLabel("PP1", "PP1")
            with app.page():
                app.addLabel("PP2", "PP2")
            with app.page():
                app.addLabel("PP3", "PP3")

    logout()



#!/usr/bin/env python3
# tkinter-png - example of using tkinter and pypng to display pngs (albeit reduced quality)
# in nothing but pure python. Can use RGBA images, but alpha is opaque or transparent only.
# v0.75 - Example code and module seperated out, speed optimisation of the convert function

from array import *
try:
    # for Python2
    from Tkinter import *
except ImportError:
    # for Python3
    from tkinter import *
from . import png

## tkinter hacking section ##

# Define our new tkinter.PhotoImage functions and override the stock version
class PhotoImage(PhotoImage):
    def transGet(self, x, y):
        """Returns a boolean if pixel at (x,y) is transparent"""
        return self.tk.call(self.name, "transparency", "get", x, y)
    
    def transSet(self, x, y, alpha):
        """Makes the pixel at (x,y) transparent if alpha is is true or opaque otherwise"""
        self.tk.call(self.name, "transparency", "set", x, y, alpha)
    
    # tkinter already has a gimped version of copy, but it's pretty useless
    # this version is still lacking and doesn't have all options, but is actually usable
    def copy(self, sourceImage, fromBox=None, toBox=None):
        """Copies from region of sourceImage at fromBox to current image at toBox"""
        args = (self.name, "copy", sourceImage)
        if fromBox:
            if fromBox[0] == "-from":
                fromBox = fromBox[1:]
            args = args + ("-from",) + tuple(fromBox)
        if toBox:
            if toBox[0] == "-to":
                toBox = toBox[1:]
            args = args + ("-to",) + tuple(toBox)
        self.tk.call(args)
    
    def redither(self):
        """Recalculate dithering used in PhotoImages to fix errors that may occur if image data was supplied in chunks"""
        self.tk.call(self.name, "redither")
    
    def data(self, bg=None, fromBox=None, grey=None):
        """Returns image data in the form of a string"""
        args = (self.name, "data")
        if bg:
            if bg[0] == "-background":
                bg = bg[1:]
            args = args + ("-background",) + tuple(bg)
        if fromBox:
            if fromBox[0] == "-from":
                fromBox = fromBox[1:]
            args = args + ("-from",) + tuple(fromBox)
        if grey:
            if grey is True or grey == "-grayscale":
                args = args + ("-grayscale",)


## PngImageTk section ##

class PngImageTk(object):
    """A png image loaded and placed into a tkinter.PhotoImage object"""
    def __init__(self, filename):
        # Read image, create list of pixel RGB or RGBA values
        r = png.Reader(filename)
        # Try to use RGB8 load if no alpha chanel otherwise use alpha (RGBA8)
        try:
            self.w, self.h, self.pixels, self.meta = r.asRGB8()
        except :
            self.w, self.h, self.pixels, self.meta = r.asRGBA8()
        self.pixeldata = list(self.pixels) #pixeldata has each row of the image as an array
        self.x = 0
        self.y = 0
        self.image = PhotoImage(width=self.w, height=self.h) #use photoimage as temporary oject to write to canvas

    # Print meta data for image
    def __str__(self):
        rep = "Width:", self.width, "\n"
        rep += "Height:", self.height, "\n"
        rep += "Bitdepth:", self.meta["bitdepth"], "\n"
        rep += "Greyscale:", self.meta["greyscale"], "\n"
        rep += "Alpha:", self.meta["alpha"], "\n"
        return rep
        
    # Used to split each row into pairs of RGB or RGBA values
    def chunks(self, l, n):
        return [l[i:i+n] for i in range(0, len(l), n)]

    # Convert pixeldata into a PhotoImage object
    def convert(self):
        alphapixels = []
        if self.meta["alpha"] is True:
            values = 4
            a_append = alphapixels.append
        else:
            values = 3
            
        pixelrows = []
        
        # Possible optimisation by minimising re-evaluation of dot notation in loops
        p_append = pixelrows.append
        pixeldata = self.pixeldata
        chunks = self.chunks
        alpha = self.meta["alpha"]
        x = self.x
        y = self.y
        w = self.w
        h = self.h
        put = self.image.put
        transSet = self.image.transSet
        
        for row in pixeldata:
            row = row.tolist() #convert from array to list
            chunked = chunks(row, values) #RGB/RGBA format = 3/4 values
                        
            for item in chunked:
                if alpha is True:
                    # if 100% transparent, remember this pixel so we can make it transparent later
                    if item[3] == 0:
                        a_append((x, y))
                    del item[-1] #remove alpha bit
                    
                    # Increment position, used for tracking coordinates of transparent pixels
                    x += 1
                    if x == w:
                        y += 1
                        x = 0
                        
            p_append(["#%02x%02x%02x" % tuple(item) for item in chunked])
        
        pixelrows = tuple(tuple(x) for x in pixelrows) #convert our list of lists into a tuple of tuples
        put(pixelrows,(0,0, w,h)) #pixels are finally written to the PhotoImage
        
        # If we have alphapixels, set each stored coordinate to transparent
        if alphapixels:
            for item in alphapixels:
                transSet(item[0],item[1], "True")



# -*- coding: utf-8 -*-
from appJar.appjar import gui
from appJar.appjar import SelectableLabel
from appJar.appjar import AutoCompleteEntry
from appJar.appjar import ajScale
from appJar.appjar import AjText, AjScrolledText
from appJar.appjar import Meter
from appJar.appjar import Properties
from appJar.appjar import Link
from appJar.appjar import Separator
from appJar.appjar import Grip
from appJar.appjar import PieChart
from appJar.appjar import DraggableWidget



from distutils.core import setup
setup(
    name="appJar",
    packages=["appJar"],
    version="0.8.0",
    description="An easy-to-use, feature-rich GUI wrapper for tkinter. Designed specifically for use in the classroom, but powerful enough to be used anywhere.",
    author="Richard Jarvis",
    author_email="info@appjar.info",
    url="http://appJar.info",
    keywords=["python", "gui", "tkinter"],
    license="Apache 2.0",
    classifiers=[
	'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Education',
        'Topic :: Software Development',
        'Topic :: Software Development :: User Interfaces',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: Apache Software License',

    ],
    package_data = {
        "appJar": ["lib/*.py", "lib/*.txt", "lib/tkdnd2.8/*.tcl", "lib/tkdnd2.8/tcl_files/*.tcl", "lib/tkdnd2.8/tcl_libs/*", "resources/icons/*", "examples/showcase.py"]
    }
)
